// Copyright 2019 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// Package genname contains constants for generated names.
package genname

const (
	State = "state"

	SizeCache  = "sizeCache"
	SizeCacheA = "XXX_sizecache"

	WeakFields  = "weakFields"
	WeakFieldsA = "XXX_weak"

	UnknownFields  = "unknownFields"
	UnknownFieldsA = "XXX_unrecognized"

	ExtensionFields  = "extensionFields"
	ExtensionFieldsA = "XXX_InternalExtensions"
	ExtensionFieldsB = "XXX_extensions"

	WeakFieldPrefix = "XXX_weak_"
)
